// ---------------------------------------------------------------------------
// Begin: Support for Class Inheritance
//
// Note: This came from http://www.crockford.com/javascript/inheritance.html
// ---------------------------------------------------------------------------

/**
 * Add an instance method to a class.
 * This adds a public method to the Function.prototype, so all functions get it by Class Augmentation.
 * It takes a name and a function, and adds them to a function's prototype object.
 * It returns this.
 */
Function.prototype.method = function(name, func) {
    this.prototype[name] = func;
    return this;
};

/**
 * The inherits method.
 * It indicates that one class inherits from another.
 * It should be called after both classes are defined, but before the inheriting class's methods are added.
 */
Function.method('inherits', function (Parent) {
    this.prototype = new Parent();
    var d = {},
        p = this.prototype;
    this.prototype.constructor = Parent;
    this.method('uber', function uber(name) {
        if (!(name in d)) {
            d[name] = 0;
        }
        var f, r, t = d[name], v = Parent.prototype;
        if (t) {
            while (t) {
                v = v.constructor.prototype;
                t -= 1;
            }
            f = v[name];
        } else {
            f = p[name];
            if (f == this[name]) {
                f = v[name];
            }
        }
        d[name] += 1;
        r = f.apply(this, Array.prototype.slice.apply(arguments, [1]));
        d[name] -= 1;
        return r;
    });
    return this;
});

/**
 * The swiss method is used to support multiple inheritance.
 */
Function.method('swiss', function (parent) {
    var ii;
    var name;

    for (ii = 1; ii < arguments.length; ii += 1) {
        name = arguments[ii];
        this.prototype[name] = parent.prototype[name];
    }
    return this;
});

// ---------------------------------------------------------------------------
// End: Support for Class Inheritance
// ---------------------------------------------------------------------------
